#version 330 core

layout (location = 0) out vec4 outputColor;

uniform vec3 CameraPosition;
uniform mat4 PerspectiveInverseMatrix;

uniform sampler2D DiffuseTexture;
uniform sampler2D NormalDepthTexture;
uniform sampler2D ShadowMap;

uniform vec2 ScreenSize;

uniform vec4 LightColor;
uniform vec3 LightDirection;
uniform float SpecularPower;
uniform mat4 SpecularIntensityMtx;
uniform float AmbientIntensity;
uniform float DiffuseIntensity;
uniform float FarClip;
uniform mat4 MVP;
uniform vec4 ProjInfo;
uniform mat4 ViewInverse;
uniform mat4 WorldToShadow;

vec2 CalcTexCoord()
{
    return gl_FragCoord.xy / ScreenSize;
}

vec3 CalcViewCoord(float inDepth)
{
    inDepth *= FarClip;
    ivec2 itc = ivec2(gl_FragCoord.xy);
    return vec3((vec2(itc) * ProjInfo.xy + ProjInfo.zw) * inDepth, inDepth);
}

float ReadShadowMap(vec3 worldPos)
{
    vec3 shadowPos = (WorldToShadow * vec4(worldPos, 1)).xyz;
    vec2 textureCoordinates = shadowPos.xy * vec2(0.5,0.5) + vec2(0.5,0.5);

    const float bias = 0.0000001;
    float depthValue = texture( ShadowMap, textureCoordinates ).r - bias;
    if(shadowPos.z * 0.5 + 0.5 < depthValue)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

vec3 CalcWorldCoord(vec3 viewPos)
{
    return (ViewInverse * vec4(viewPos.xyz, 1)).xyz;
}

vec4 CalcLightInternal(vec3 color,
                       float ambientIntensity,
                       float diffuseIntensity,
					   vec3 lightDirection,
					   vec3 worldPos,
					   vec3 normal)
{
    vec4 ambientColor = vec4(color, 1.0) * ambientIntensity;
    float shadow = ReadShadowMap(worldPos);
    float diffuseFactor = dot(normal, -lightDirection) * shadow;

    vec4 diffuseColor  = vec4(0, 0, 0, 0);
    vec4 specularColor = vec4(0, 0, 0, 0);

    if (diffuseFactor > 0.0) {
        diffuseColor = vec4(color, 1.0) * diffuseIntensity * diffuseFactor;

        vec3 vertexToEye = normalize(CameraPosition - worldPos);
        vec3 lightReflect = normalize(reflect(lightDirection, normal));
        float specularFactor = dot(vertexToEye, lightReflect);
        specularFactor = pow(specularFactor, SpecularPower);
        if (specularFactor > 0.0) {
            //specularColor = vec4(color, 1.0) * SpecularIntensityMtx * specularFactor;
        }
    }

    return (ambientColor + diffuseColor + specularColor);
}

vec4 CalcDirectionalLight(vec3 worldPos, vec3 normal)
{
    return CalcLightInternal(LightColor.rgb,
                             AmbientIntensity,
                             0.3, //DiffuseIntensity,
							 LightDirection.xyz,
							 worldPos,
							 normal);
}

/* vec4 CalcPointLight(vec3 WorldPos, vec3 Normal) */
/* { */
/*     vec3 LightDirection = WorldPos - gPointLight.Position; */
/*     float Distance = length(LightDirection); */
/*     LightDirection = normalize(LightDirection); */

/*     vec4 Color = CalcLightInternal(gPointLight.Base, LightDirection, WorldPos, Normal); */

/*     float attenuation =  gPointLight.Atten.Constant + */
/*                          gPointLight.Atten.Linear * Distance + */
/*                          gPointLight.Atten.Exp * Distance * Distance; */

/*     attenuation = max(1.0, attenuation); */

/*     return Color / Attenuation; */
/* } */


void main( void )
{
    vec2 UV = CalcTexCoord();
    vec4 normalDepth = texture(NormalDepthTexture, UV);
    vec3 position = CalcWorldCoord(CalcViewCoord(normalDepth.w));

    vec4 diffuse = texture(DiffuseTexture, UV);
    vec3 normal = normalize(normalDepth.xyz);

    outputColor = diffuse * CalcDirectionalLight(position, normal.xyz);
    //outputColor = WorldToShadow * vec4(position, 1);
    //outputColor = position;
    //outputColor = diffuse;
    //outputColor = vec4(UV.x, UV.y,0,1);
    //outputColor = vec4(0,0,0,0);
}

