#version 330 core

layout(location = 0) out vec4 outputColor;

uniform vec3 CameraPosition;

uniform sampler2D DiffuseTexture;
uniform sampler2D NormalDepthTexture;
uniform sampler2D PositionTexture;

uniform vec2 ScreenSize;
uniform mat4 PerspectiveInverseMatrix;
uniform mat4 ViewMatrix;
uniform mat4 ViewInverse;
uniform mat4 World;

uniform vec3 LightPosition;
uniform vec4 LightColor;
uniform float SpecularPower;
uniform mat4 SpecularIntensityMtx;
uniform float AmbientIntensity;
uniform float DiffuseIntensity;

uniform float AttenuationConstant;
uniform float AttenuationLinear;
uniform float AttenuationExponent;
uniform float FarClip;
uniform vec4 ProjInfo;

vec2 CalcTexCoord()
{
    return gl_FragCoord.xy / ScreenSize;
}

vec3 CalcViewCoord(vec2 UV, float inDepth)
{
    inDepth *= FarClip;
    ivec2 itc = ivec2(gl_FragCoord.xy);
    return vec3((vec2(itc) * ProjInfo.xy + ProjInfo.zw) * inDepth, inDepth);
}

vec3 CalcWorldPos(vec3 viewPos)
{
    return (ViewInverse * vec4(viewPos.xyz, 1)).xyz;
}

vec4 CalcLightInternal(vec3 color,
                       float ambientIntensity,
                       float diffuseIntensity,
					   vec3 lightDirection,
					   vec3 viewPos,
					   vec3 normal)
{
    vec4 ambientColor = vec4(color, 1.0) * ambientIntensity;
    float diffuseFactor = dot(normal, -lightDirection);

    vec4 diffuseColor  = vec4(0, 0, 0, 0);
    vec4 specularColor = vec4(0, 0, 0, 0);

    if (diffuseFactor > 0.0) {
        diffuseColor = vec4(color, 1.0) * diffuseIntensity * diffuseFactor;

        vec3 vertexToEye = normalize(CameraPosition - viewPos);
        vec3 lightReflect = normalize(reflect(lightDirection, normal));
        float specularFactor = dot(vertexToEye, lightReflect);
        specularFactor = pow(specularFactor, SpecularPower);
        if (specularFactor > 0.0) {
            specularColor = vec4(color, 1.0) * SpecularIntensityMtx * specularFactor;
        }
    }

    return (ambientColor + diffuseColor + specularColor);
}

vec4 CalcPointLight(vec3 viewPos, vec3 normal)
{
    vec3 lightPosition = LightPosition;
    vec3 LightDirection = viewPos - lightPosition;
    float dist = length(LightDirection);
    LightDirection = normalize(LightDirection);

    vec4 lightColor = LightColor;
    vec4 Color = CalcLightInternal(lightColor.rgb,
                                   AmbientIntensity,
                                   DiffuseIntensity,
                                   LightDirection.xyz,
                                   viewPos, normal);

    float constant = 1;
    float linear = 0.5;
    float exp = 0.03;

    float attenuation =  constant +
        linear * dist +
        exp * dist * dist;

    attenuation = max(1.0, attenuation);

    return Color / attenuation;
}

void main( void )
{
    vec2 UV = CalcTexCoord();

    vec4 diffuse = texture(DiffuseTexture, UV);
    vec4 normalDepth = texture(NormalDepthTexture, UV);
    vec3 position = CalcWorldPos(CalcViewCoord(UV, normalDepth.w));
    vec3 normal = normalize(normalDepth.xyz);

    outputColor = diffuse * CalcPointLight(position, normal.xyz);
}

